<?php

namespace Drupal\dblog_ban\Controller;

use Drupal\ban\BanIpManagerInterface;
use Drupal\Core\Ajax\AfterCommand;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\dblog_ban\Services\BanLinkGenerator;
use Drupal\dblog_ban\Services\IpValidator;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Respond to AJAX requests to ban and unban IP addresses.
 */
class IpBanUnbanAjaxController extends ControllerBase {

  /**
   * A ban IP manager.
   *
   * @var \Drupal\ban\BanIpManagerInterface
   */
  private $banIpManager;

  /**
   * Service to generate ban and unban links.
   *
   * @var \Drupal\dblog_ban\Services\BanLinkGenerator
   */
  private $banLinkGenerator;

  /**
   * Service to validate hostnames or IP addresses.
   *
   * @var \Drupal\dblog_ban\Services\IpValidator
   */
  private $ipValidator;

  /**
   * Constructs a IpBanUnbanAjaxController object.
   *
   * @param \Drupal\ban\BanIpManagerInterface $banIpManager
   *   A ban IP manager.
   * @param \Drupal\dblog_ban\Services\BanLinkGenerator $banLinkGenerator
   *   Service to generate ban and unban links.
   * @param \Drupal\dblog_ban\Services\IpValidator $ipValidator
   *   Service to validate hostnames or IP addresses.
   */
  public function __construct(BanIpManagerInterface $banIpManager, BanLinkGenerator $banLinkGenerator, IpValidator $ipValidator) {
    $this->banIpManager = $banIpManager;
    $this->banLinkGenerator = $banLinkGenerator;
    $this->ipValidator = $ipValidator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('ban.ip_manager'),
      $container->get('dblog_ban.link_generator'),
      $container->get('dblog_ban.ip_validator')
    );
  }

  /**
   * Respond to an AJAX request to ban an IP address.
   *
   * @param string $ip
   *   The IP address to ban.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An AjaxResponse to replace the ban link with an unban link.
   */
  public function ajaxBan(string $ip): AjaxResponse {
    $response = new AjaxResponse();
    $rawIp = \urldecode($ip);

    if ($this->ipValidator->isValidIp($rawIp) && !$this->ipValidator->isMyIp($rawIp)) {
      $this->banIpManager->banIp($rawIp);

      // Note that, due to a bug in jQuery <1.9, ajax_command_replace() will
      // only replace one link; and delete the others
      // (https://github.com/jquery/jquery/pull/920), so we get around this by
      // inserting the new ban link, then deleting the old unban link.
      $response->addCommand(new AfterCommand("[data-dblog_ban-ip='$rawIp'][data-dblog_ban-action='ban']",
        $this->banLinkGenerator->getUnbanLink($rawIp)->toString()));
      $response->addCommand(new RemoveCommand("[data-dblog_ban-ip='$rawIp'][data-dblog_ban-action='ban']"));
    }
    return $response;
  }

  /**
   * Respond to an AJAX request to unban an IP address.
   *
   * @param string $ip
   *   The IP address to unban.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   An AjaxResponse to replace the unban link with a ban link.
   */
  public function ajaxUnban(string $ip): AjaxResponse {
    $response = new AjaxResponse();
    $rawIp = \urldecode($ip);

    if ($this->ipValidator->isValidIp($rawIp)) {
      $this->banIpManager->unbanIp($rawIp);

      // Note that, due to a bug in jQuery <1.9, ajax_command_replace() will
      // only replace one link; and delete the others
      // (https://github.com/jquery/jquery/pull/920), so we get around this by
      // inserting the new ban link, then deleting the old unban link.
      $response->addCommand(new AfterCommand("[data-dblog_ban-ip='$rawIp'][data-dblog_ban-action='unban']",
        $this->banLinkGenerator->getBanLink($rawIp)->toString()));
      $response->addCommand(new RemoveCommand("[data-dblog_ban-ip='$rawIp'][data-dblog_ban-action='unban']"));
    }
    return $response;
  }

}
